"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var icss_utils_1 = require("icss-utils");
var postcss = require("postcss");
var postcssIcssSelectors = require("postcss-icss-selectors");
var classTransforms_1 = require("./classTransforms");
var processor = postcss(postcssIcssSelectors({ mode: 'local' }));
exports.getClasses = function (css) {
    return icss_utils_1.extractICSS(processor.process(css).root).icssExports;
};
var classNameToProperty = function (className) { return "'" + className + "': string;"; };
var classNameToUnion = function (className) { return "'" + className + "'"; };
var flattenClassNames = function (previousValue, currentValue) {
    if (previousValue === void 0) { previousValue = []; }
    return previousValue.concat(currentValue);
};
var classnamesTypes = function (classNames) { return "\ntype ClassKeys = " + classNames.map(classNameToUnion).join(' | ') + "\ntype ClassDictionary = { [id in ClassKeys]?: any };\ntype ClassValue = ClassKeys | ClassDictionary;\n\ninterface ClassArray extends Array<ClassValue> {}\ninterface ClassNames extends ClassDictionary {\n  (...classes: ClassValue[]) => string;\n}\n\ndeclare const classNamesFn: ClassNames;\n\nexport default classNamesFn"; };
exports.createExports = function (classes, options) {
    var classNames = Object.keys(classes)
        .map(classTransforms_1.transformClasses(options.camelCase))
        .reduce(flattenClassNames);
    return "declare const classes: {\n  " + classNames.map(classNameToProperty).join('\n  ') + "\n};\n" + (options.classnamesLoader ? classnamesTypes : 'export default classes') + ";\n";
};
exports.getDtsSnapshot = function (ts, scriptSnapshot, options) {
    var css = scriptSnapshot.getText(0, scriptSnapshot.getLength());
    var classes = exports.getClasses(css);
    var dts = exports.createExports(classes, options);
    return ts.ScriptSnapshot.fromString(dts);
};
